function [xmin, fmin, FES, func_near, std, fmin_conv] =cuckoo(fn_name, a, b, N, my_eps, max_evals, f_near, pa, shift)
%vystup:
%xmin = souadnice nalezenho een
%fmin = funkn hodnota nalezenho een
%FES = poet evaluac funkce
%func_near = evaluac kdy se pekroilo f_near
%std = pole smrodatnch odchylek v 10 etapch 
%fmin_conv = vvoj nalezenho minima, 1 inicializace, 10 max evaluac

%vstup:
%a, b = spodn a horn hranice prohledvanho prostoru
%N = velikost populace
% my_eps = ukon. podmnka kdy je populace blzko u sebe 
% max_evals = max evaluac
%f_near = podmnka pro func_near
%pa =  parametry algoritmu, shift je posun 

D = length(a);
nest = zeros(N, D);

%ukladani prubehu vypoctu
FES_lim = zeros(10, 1);
for i = 2:10
    l = i-1;
    FES_lim(i) = l * 0.1 * max_evals;
end
fmin_conv = NaN(10,1);
std = NaN(10,1);
etapa = 1;

% Random initial solutions
for i=1:N,
nest(i, 1:D)= a + (b - a) .* rand(1, D);
end

% Get the current best
fitness=10^10*ones(N,1);
[fmin,xmin,nest,fitness, fmax]=get_best_nest(nest,nest,fitness, fn_name, shift);
FES=N;
func_near=0;

%main loop
while (fmax - fmin > my_eps) && (FES < max_evals) && (fmin > 1e-8)
    % Generate new solutions (but keep the current best)
     new_nest=get_cuckoos(nest,xmin,a,b);   
     [fnew,best,nest,fitness]=get_best_nest(nest,new_nest,fitness, fn_name, shift);
    % Update the counter
      FES=FES+N; 
    % Discovery and randomization
      new_nest=empty_nests(nest,a,b,pa) ;
    
    % Evaluate this set of solutions
      [fnew,best,nest,fitness]=get_best_nest(nest,new_nest,fitness, fn_name, shift);
    % Update the counter again
      FES=FES+N;
    % Find the best objective so far  
    if fnew<fmin,
        fmin=fnew;
        xmin=best;
    end
    
    if func_near == 0 && fmin < f_near
        func_near = FES;
    end  
    
    if (fmin < 1e-8)
        fmin = 1e-8;
    end
    
     %vypocet sm odchylky a prubeh hledani
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    prum = mean(nest(:,1:D)); %stred prostoru
    prum_mat = repmat(prum, N, 1); %sted maticov
    
    if etapa < 10 && isnan(fmin_conv(etapa)) && FES >= FES_lim(etapa)
        fmin_conv(etapa) = fmin;
        std(etapa) = sqrt(sum(sum((nest(:, 1:D) - prum_mat).*(nest(:,1:D) - prum_mat)))/N); %etapa = 1..10
        etapa = etapa + 1;
    end
end
%end while


%%%%%%%%%%%% 
%% --------------- All subfunctions are list below ------------------
%% Get cuckoos by ramdom walk
function nest=get_cuckoos(nest,best,Lb,Ub)
% Levy flights
n=size(nest,1);
% Levy exponent and coefficient
% For details, see equation (2.21), Page 16 (chapter 2) of the book
% X. S. Yang, Nature-Inspired Metaheuristic Algorithms, 2nd Edition, Luniver Press, (2010).
beta=3/2;
sigma=(gamma(1+beta)*sin(pi*beta/2)/(gamma((1+beta)/2)*beta*2^((beta-1)/2)))^(1/beta);

for j=1:n,
    s=nest(j,:);
    % This is a simple way of implementing Levy flights
    % For standard random walks, use step=1;
    %% Levy flights by Mantegna's algorithm
    u=randn(size(s))*sigma;
    v=randn(size(s));
    step=u./abs(v).^(1/beta);
  
    % In the next equation, the difference factor (s-best) means that 
    % when the solution is the best solution, it remains unchanged.     
    stepsize=0.01*step.*(s-best);
    % Here the factor 0.01 comes from the fact that L/100 should the typical
    % step size of walks/flights where L is the typical lenghtscale; 
    % otherwise, Levy flights may become too aggresive/efficient, 
    % which makes new solutions (even) jump out side of the design domain 
    % (and thus wasting evaluations).
    % Now the actual random walks or flights
    s=s+stepsize.*randn(size(s));
   % Apply simple bounds/limits
   nest(j,:)=simplebounds(s,Lb,Ub);
end

%% Find the current best nest
function [fmin, best,nest,fitness, fmax]=get_best_nest(nest,newnest,fitness, fn_name, shift)
% Evaluating all new solutions
for j=1:size(nest,1),
    fnew=feval(fn_name, newnest(j, :) - shift);
    
    if fnew<=fitness(j),
       fitness(j)=fnew;
       nest(j,:)=newnest(j,:);
    end
end
% Find the current best
[fmin,K]=min(fitness) ;
fmax = max(fitness);
best=nest(K,:);

%% Replace some nests by constructing new solutions/nests
function new_nest=empty_nests(nest,Lb,Ub,pa)
% A fraction of worse nests are discovered with a probability pa
n=size(nest,1);
% Discovered or not -- a status vector
K=rand(size(nest))>pa;

% In the real world, if a cuckoo's egg is very similar to a host's eggs, then 
% this cuckoo's egg is less likely to be discovered, thus the fitness should 
% be related to the difference in solutions.  Therefore, it is a good idea 
% to do a random walk in a biased way with some random step sizes.  
%% New solution by biased/selective random walks
stepsize=rand*(nest(randperm(n),:)-nest(randperm(n),:));
new_nest=nest+stepsize.*K;
for j=1:size(new_nest,1)
    s=new_nest(j,:);
  new_nest(j,:)=simplebounds(s,Lb,Ub);  
end

% Application of simple constraints
function s=simplebounds(s,Lb,Ub)
  % Apply the lower bound
  ns_tmp=s;
  I=ns_tmp<Lb;
  ns_tmp(I)=Lb(I);
  
  % Apply the upper bounds 
  J=ns_tmp>Ub;
  ns_tmp(J)=Ub(J);
  % Update this new move 
  s=ns_tmp;





